/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2008 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.tools.webadmin.datacollector.client.service;

import java.util.Map;
import java.util.Set;

import javax.management.ObjectName;

import org.ow2.petals.jmx.PetalsComponentClient;
import org.ow2.petals.jmx.PetalsJMXClient;
import org.ow2.petals.jmx.exception.AdminDoesNotExistException;
import org.ow2.petals.jmx.exception.AdminServiceErrorException;
import org.ow2.petals.jmx.exception.AttributeErrorException;
import org.ow2.petals.jmx.exception.ComponentDoesNotExistException;
import org.ow2.petals.jmx.exception.ComponentErrorException;
import org.ow2.petals.jmx.exception.ConnectionErrorException;
import org.ow2.petals.jmx.exception.PerformActionErrorException;
import org.ow2.petals.jmx.exception.PetalsAdminDoesNotExistException;
import org.ow2.petals.jmx.exception.PetalsAdminServiceErrorException;
import org.ow2.petals.tools.webadmin.datacollector.client.exception.DataCollectorClientException;

/**
 * @author mcarpentier - EBM WebSourcing
 * 
 * Deals with petals admin functions
 * 
 */
public class AdminService extends CollectorService {

    public AdminService(PetalsJMXClient petalsJMXClient) {
        super(petalsJMXClient);
    }

    public Set<Map<String, String>> getTopology() throws DataCollectorClientException {
        try {
            final Set<Map<String, String>> topology = this.getPetalsJMXClient()
                    .getPetalsAdminServiceClient().retrieveTopology();
            return topology;
        } catch (final PetalsAdminServiceErrorException e) {
            throw new DataCollectorClientException("Can't get PEtALS topology", e);
        } catch (final PetalsAdminDoesNotExistException e) {
            throw new DataCollectorClientException("Can't get PEtALS topology", e);
        } catch (final ConnectionErrorException e) {
            throw new DataCollectorClientException("Can't get PEtALS topology", e);
        }
    }

    public String getSystemInfo() throws DataCollectorClientException {
        try {
            return this.getPetalsJMXClient().getAdminServiceClient().getSystemInfo();
        } catch (AdminDoesNotExistException e) {
            throw new DataCollectorClientException("Can't retrieve version of PEtALS", e);
        } catch (AdminServiceErrorException e) {
            throw new DataCollectorClientException("Can't retrieve version of PEtALS", e);
        } catch (ConnectionErrorException e) {
            throw new DataCollectorClientException("Can't retrieve version of PEtALS", e);
        } catch (DataCollectorClientException e) {
            throw new DataCollectorClientException("Can't retrieve version of PEtALS", e);
        }
    }

    public ObjectName getComponentByName(String componentName) throws DataCollectorClientException {
        try {

            final ObjectName result = this.getPetalsJMXClient().getAdminServiceClient()
                    .getComponentByName(componentName);

            return result;

        } catch (final AdminServiceErrorException e) {
            throw new DataCollectorClientException("Can't get component '" + componentName
                    + "' from petals", e);
        } catch (final AdminDoesNotExistException e) {
            throw new DataCollectorClientException("Can't find PetalsAdmin service", e);
        } catch (final ConnectionErrorException e) {
            throw new DataCollectorClientException(e);
        }
    }

    public ObjectName[] getBindingComponents() throws DataCollectorClientException {
        try {
            final ObjectName[] result = this.getPetalsJMXClient().getAdminServiceClient()
                    .getBindingComponents();

            return result;

        } catch (final AdminServiceErrorException e) {
            throw new DataCollectorClientException("Can't retrieve binding components from petals",
                    e);
        } catch (final AdminDoesNotExistException e) {
            throw new DataCollectorClientException("Can't find PetalsAdmin service", e);
        } catch (final ConnectionErrorException e) {
            throw new DataCollectorClientException(e);
        }

    }

    public String getComponentState(String componentName) throws DataCollectorClientException {
        try {

            final PetalsComponentClient pccc = this.getPetalsJMXClient()
                    .getComponentClient(componentName);

            return pccc.getState();

        } catch (final ComponentDoesNotExistException e) {
            throw new DataCollectorClientException(e);
        } catch (final ComponentErrorException e) {
            throw new DataCollectorClientException(e);
        } catch (final ConnectionErrorException e) {
            throw new DataCollectorClientException(e);
        }
    }

    public ObjectName[] getEngineComponents() throws DataCollectorClientException {
        try {
            final ObjectName[] result = this.getPetalsJMXClient().getAdminServiceClient()
                    .getEngineComponents();

            return result;

        } catch (final AdminServiceErrorException e) {
            throw new DataCollectorClientException("Can't retrieve engine components from petals",
                    e);
        } catch (final AdminDoesNotExistException e) {
            throw new DataCollectorClientException("Can't find PetalsAdmin service", e);
        } catch (final ConnectionErrorException e) {
            throw new DataCollectorClientException(e);
        }

    }

    public void shutdownComponent(String componentName) throws DataCollectorClientException {
        try {

            PetalsComponentClient pccc = null;

            // We check if the component exists
            pccc = this.getPetalsJMXClient().getComponentClient(componentName);

            if (PetalsComponentClient.State.STOPPED.equals(pccc.getState())) {
                pccc.shutdown();
            } else {
                throw new DataCollectorClientException("Can't perform start action, the component "
                        + componentName + " is in the following state: " + pccc.getState());
            }

        } catch (final ComponentDoesNotExistException e) {
            throw new DataCollectorClientException(e);
        } catch (final ComponentErrorException e) {
            throw new DataCollectorClientException(e);
        } catch (final ConnectionErrorException e) {
            throw new DataCollectorClientException(e);
        }
    }

    public void startComponent(String componentName) throws DataCollectorClientException {
        try {

            PetalsComponentClient pccc = null;

            // We check if the component exists
            pccc = this.getPetalsJMXClient().getComponentClient(componentName);

            if (PetalsComponentClient.State.STARTED.equals(pccc.getState())) {
                // The component is already started
                throw new DataCollectorClientException("The component is already started.");
            }

            if ((PetalsComponentClient.State.SHUTDOWN.equals(pccc.getState()))
                    || (PetalsComponentClient.State.STOPPED.equals(pccc.getState()))) {
                // Starting the component

                pccc.start();
            } else {
                throw new DataCollectorClientException("Can't perform start action, the component "
                        + componentName + " is in the following state: " + pccc.getState());
            }

        } catch (final ComponentDoesNotExistException e) {
            throw new DataCollectorClientException(e);
        } catch (final ComponentErrorException e) {
            throw new DataCollectorClientException(e);
        } catch (final ConnectionErrorException e) {
            throw new DataCollectorClientException(e);
        }
    }

    public void stopComponent(String componentName) throws DataCollectorClientException {
        try {

            PetalsComponentClient pccc = null;

            // We check if the component exists
            pccc = this.getPetalsJMXClient().getComponentClient(componentName);

            if (PetalsComponentClient.State.STARTED.equals(pccc.getState())) {
                pccc.stop();
            } else {
                throw new DataCollectorClientException("Can't perform stop action, the component "
                        + componentName + " is in the following state: " + pccc.getState());
            }

        } catch (final ComponentDoesNotExistException e) {
            throw new DataCollectorClientException(e);
        } catch (final ComponentErrorException e) {
            throw new DataCollectorClientException(e);
        } catch (final ConnectionErrorException e) {
            throw new DataCollectorClientException(e);
        }
    }

}
