/**
 * PETALS - PETALS Services Platform. Copyright (c) 2008 EBM WebSourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.tools.webadmin.datacollector.client.service;

import java.net.URL;

import org.ow2.petals.jmx.PetalsDeploymentServiceClient;
import org.ow2.petals.jmx.PetalsInstallationServiceClient;
import org.ow2.petals.jmx.PetalsInstallerComponentClient;
import org.ow2.petals.jmx.PetalsJMXClient;
import org.ow2.petals.jmx.exception.ConnectionErrorException;
import org.ow2.petals.jmx.exception.DeploymentServiceErrorException;
import org.ow2.petals.jmx.exception.InstallationServiceDoesNotExistException;
import org.ow2.petals.jmx.exception.InstallationServiceErrorException;
import org.ow2.petals.jmx.exception.PerformActionErrorException;
import org.ow2.petals.tools.webadmin.datacollector.client.exception.DataCollectorClientException;

/**
 * @author Adrien LOUIS - EBMWebSourcing
 * @author mcarpentier - EBM WebSourcing
 * 
 *         Deals with petals installation functions
 * 
 */
public class InstallService extends CollectorService {

    public InstallService(PetalsJMXClient petalsJMXClient) {
        super(petalsJMXClient);

    }

    public void installComponent(final URL fileURL) throws DataCollectorClientException {

        try {
            final PetalsInstallerComponentClient picc = this.getPetalsJMXClient()
                    .getInstallationServiceClient().loadNewInstaller(fileURL);
            picc.install();
        } catch (final PerformActionErrorException e) {
            throw new DataCollectorClientException(e);
        } catch (final InstallationServiceDoesNotExistException e) {
            throw new DataCollectorClientException("Unable to find Installation service", e);
        } catch (final InstallationServiceErrorException e) {
            throw new DataCollectorClientException(e);
        } catch (final ConnectionErrorException e) {
            throw new DataCollectorClientException(e);
        }
    }

    public void uninstallComponent(final String componentName) throws DataCollectorClientException {

        try {
            PetalsInstallationServiceClient pisc = this.getPetalsJMXClient()
                    .getInstallationServiceClient();

            final PetalsInstallerComponentClient picc = this.getPetalsJMXClient()
                                                        .getInstallationServiceClient()
                                                        .loadInstaller(componentName);

            if (picc.isInstalled()) {
                picc.uninstall();
                pisc.unloadInstaller(componentName);
            } else {
                throw new DataCollectorClientException("Component is not installed" + componentName);
            }
        } catch (final ConnectionErrorException e) {
            throw new DataCollectorClientException(e);
        } catch (final PerformActionErrorException e) {
            throw new DataCollectorClientException(e);
        } catch (InstallationServiceDoesNotExistException e) {
            throw new DataCollectorClientException(e);
        } catch (InstallationServiceErrorException e) {
            throw new DataCollectorClientException(e);
        }
    }

    public String[] retrieveInstalledSharedLibraries() throws DataCollectorClientException {
        try {
            return this.getPetalsJMXClient().getInstallationServiceClient()
                    .getInstalledSharedLibraries();
        } catch (InstallationServiceDoesNotExistException e) {
            throw new DataCollectorClientException("Unable to find installation service", e);
        } catch (InstallationServiceErrorException e) {
            throw new DataCollectorClientException(e);
        } catch (ConnectionErrorException e) {
            throw new DataCollectorClientException(e);
        } catch (DataCollectorClientException e) {
            throw new DataCollectorClientException(e);
        }
    }

    public void installSharedLibrary(URL sharedLibraryURL) throws DataCollectorClientException {
        try {
            this.getPetalsJMXClient().getInstallationServiceClient().installSharedLibrary(
                    sharedLibraryURL);
        } catch (InstallationServiceDoesNotExistException e) {
            throw new DataCollectorClientException("Unable to find installation service", e);
        } catch (InstallationServiceErrorException e) {
            throw new DataCollectorClientException(e);
        } catch (ConnectionErrorException e) {
            throw new DataCollectorClientException(e);
        } catch (DataCollectorClientException e) {
            throw new DataCollectorClientException(e);
        }
    }

    public boolean uninstallSharedLibrary(String sharedLibraryName)
            throws DataCollectorClientException {
        try {
            return getPetalsJMXClient().getInstallationServiceClient().uninstallSharedLibrary(
                    sharedLibraryName);
        } catch (InstallationServiceDoesNotExistException e) {
            throw new DataCollectorClientException("Unable to find installation service", e);
        } catch (InstallationServiceErrorException e) {
            throw new DataCollectorClientException(e);
        } catch (ConnectionErrorException e) {
            throw new DataCollectorClientException(e);
        } catch (DataCollectorClientException e) {
            throw new DataCollectorClientException(e);
        }
    }

    private void shutdownDeployedSA(final String serviceAssemblyName,
            PetalsDeploymentServiceClient pdsc) throws DataCollectorClientException {
        try {
            // We check its state to know if we can stop it
            if (PetalsDeploymentServiceClient.State.STARTED.equals(pdsc
                    .getState(serviceAssemblyName))) {
                // We can stop it

                pdsc.stop(serviceAssemblyName);

            }
            if (PetalsDeploymentServiceClient.State.STOPPED.equals(pdsc
                    .getState(serviceAssemblyName))) {
                // We can shutdown it

                pdsc.shutdown(serviceAssemblyName);

            }

            if (!PetalsDeploymentServiceClient.State.SHUTDOWN.equals(pdsc
                    .getState(serviceAssemblyName))) {
                throw new DataCollectorClientException("Can't shutdown the service assembly "
                        + serviceAssemblyName + " it stays in the following state: "
                        + pdsc.getState(serviceAssemblyName));
            }
        } catch (DeploymentServiceErrorException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }
}
