/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2008 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.tools.webadmin.datacollector.dao;

import java.io.Serializable;
import java.util.List;

import org.hibernate.ObjectNotFoundException;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.Transaction;
import org.ow2.petals.tools.webadmin.datacollector.util.HibernateHelper;

/**
 * @author mcarpentier
 * 
 */
public class GenericDAO {

    public void create(Session session, Object o) throws DAOException {
        Transaction tx = null;
        try {
            tx = session.beginTransaction();
            session.save(o);
            tx.commit();
        } catch (final Exception e) {
            if (tx != null) {
                tx.rollback();
            }
            if (o != null) {
                throw new DAOException("Error creating new " + o.getClass().getName(), e);
            }
            throw new DAOException("Error creating new Object of type null", e);
        }
    }

    public void delete(Object o) throws DAOException {
        Session session = null;
        try {
            session = HibernateHelper.getSession();
        } catch (final ExceptionInInitializerError e) {
            throw new DAOException("Error in Hibernate Session instanciation", e);
        } catch (final Exception e) {
            throw new DAOException("Error in Hibernate Session instanciation", e);
        }

        Transaction tx = null;
        try {
            tx = session.beginTransaction();
            session.delete(o);
            tx.commit();
        } catch (final Exception e) {
            if (tx != null) {
                tx.rollback();
            }
            if (o != null) {
                throw new DAOException("Error deleting " + o.getClass().getName(), e);
            }
            throw new DAOException("Error deleting Object of type null", e);
        }
    }

    public List<?> retrieveAll(Session session, Class<?> clazz) throws DAOException {
        try {
            final Query qr = session.createQuery("from " + clazz.getName());
            if (qr != null) {
                return qr.list();
            } else {
                return null;
            }
        } catch (final Exception e) {
            if (clazz != null) {
                throw new DAOException("Error retrieving all" + clazz.getName(), e);
            }
            throw new DAOException("Error retrieving all Objects of type null", e);
        }
    }

    public Object retrieveById(Session session, Class<?> clazz, Serializable s) throws DAOException {
        Object result;
        try {
            result = session.load(clazz, s);
        } catch (final ObjectNotFoundException e) {
            throw new DAOException("Error loading " + clazz.getName() + " with id : " + s, e);
        } catch (final Throwable e) {
            if (clazz != null) {
                throw new DAOException("Error loading " + clazz.getName() + " with id : " + s, e);
            }
            throw new DAOException("Error retrieving an Object of type null", e);
        }

        return result;
    }

    public void update(Session session, Object o) throws DAOException {
        Transaction tx = null;
        try {
            tx = session.beginTransaction();
            session.update(o);
            tx.commit();
        } catch (final Exception e) {
            if (tx != null) {
                tx.rollback();
            }
            if (o != null) {
                throw new DAOException("Error updating " + o.getClass().getName(), e);
            }
            throw new DAOException("Error updating Object of type null", e);
        }
    }

}
