/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2008 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: CollectorThread.java msauvage $
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.tools.webadmin.datacollector.util;

import org.ow2.petals.tools.webadmin.datacollector.server.CollectorServer;

/**
 * @author mcarpentier
 * 
 * This thread checks periodically the status of all queues.
 * 
 */
public class CollectorThread extends Thread {

    //private static Logger log = Logger.getLogger(CollectorThread.class);

    private int refreshFrequency = 10000;

    private CollectorServer cs;

    private boolean isRunning = false;

    public CollectorThread(CollectorServer cs) {
        this.cs = cs;
        this.refreshFrequency = cs.getQueuesCollectingInterval();
        this.setDaemon(true);
    }

    @Override
    public void run() {
        this.isRunning = true;
        while (this.isRunning) {
            cs.collectAllQueues();
            try {
                synchronized (this) {
                    this.wait(this.refreshFrequency);
                }
            } catch (final InterruptedException ie) {
                this.isRunning = false;
            }
        }
    }

    public int getRefreshFrequency() {
        return refreshFrequency;
    }

    public void setRefreshFrequency(int refreshFrequency) {
        this.refreshFrequency = refreshFrequency;
    }
}
