/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2008 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.tools.webadmin.datacollector.util;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * @author mcarpentier
 * 
 */
public class DataCollectorConfiguration {
    private final Document document;

    public DataCollectorConfiguration(Document doc) {
        this.document = doc;
    }

    public Node getComponent(String component) throws DataCollectorConfigException {
        final NodeList list = this.document.getElementsByTagName(component);
        if (list.getLength() > 1) {
            throw new DataCollectorConfigException("Multiple " + component + " nodes found ");
        }
        if (list.getLength() < 1) {
            throw new DataCollectorConfigException("No " + component + " node found");
        }
        return list.item(0);
    }

    /**
     * @param the
     *            component : datacollector-configuration,...
     * @param the
     *            property to retrieve
     * @return a Properties object containing all properties name and value
     * @throws Exception
     *             if XML document doesn't follow the associated DTD or if an
     *             error occured while parsing the configuration file
     */
    public String getProperty(String component, String property)
            throws DataCollectorConfigException {
        String result = "";
        final Node racine = this.getComponent(component);
        final NodeList nodeList = racine.getChildNodes();
        for (int i = 0; i < nodeList.getLength(); i++) {
            final Node n = nodeList.item(i);
            if (n instanceof Element) {
                if (n.getNodeName().equalsIgnoreCase(property)) {
                    final Node nn = n.getLastChild();
                    if (nn != null) {
                        result = nn.getNodeValue();
                    }
                }
            }
        }
        return result;
    }

}
