/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2008 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.tools.webadmin.datacollector.util;

import java.util.Properties;

import org.apache.log4j.Logger;
import org.hibernate.HibernateException;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.cfg.AnnotationConfiguration;
import org.ow2.petals.tools.webadmin.datacollector.bo.message.MessageExchangeSnapshot;
import org.ow2.petals.tools.webadmin.datacollector.bo.queues.ComponentQueuesStatusSnapshot;
import org.ow2.petals.tools.webadmin.datacollector.bo.queues.QueuesStatus;

/**
 * @author mcarpentier
 * 
 *         Deals with all hibernate session configuration. Can be improved by
 *         loading properties from a file...
 * 
 */
public class HibernateHelper implements IHibernateHelper {

    private static final String DATABASE_URL= "database-url";
    private static final String DATABASE_USER= "database-user";
    private static final String DATABASE_PASSWORD= "database-password";
    private static final String DATABASE_DRIVER= "database-driver";
    private static final String HIBERNATE_DIALECT= "hibernate-dialect";
    
    private static SessionFactory sessionFactory = null;

    final static Logger log = Logger.getLogger(HibernateHelper.class);

    /**
     * @return
     * @throws HibernateException
     * @throws DataCollectorConfigException
     */
    public static Session getSession() throws HibernateException, DataCollectorConfigException {
        if (HibernateHelper.sessionFactory == null) {
            log.debug("hibernate session not configured, shouldn't happen");
            //configureSession();
        }
        return HibernateHelper.sessionFactory.openSession();
    }

    /**
     * configure the hibernate session
     * 
     * @param driver
     * @param url
     * @param username
     * @param password
     */
    public static void configureSession(Properties databaseProperties) { 
        
        try {
            final Properties props = new Properties();
            
            String url = databaseProperties.getProperty(DATABASE_URL);
            String user = databaseProperties.getProperty(DATABASE_USER);
            String password = databaseProperties.getProperty(DATABASE_PASSWORD);
            String driver = databaseProperties.getProperty(DATABASE_DRIVER);
            String dialect = databaseProperties.getProperty(HIBERNATE_DIALECT); 
            
            props.setProperty("show_sql", "false");
            props.setProperty("hibernate.connection.driver_class", driver);
            props.setProperty("hibernate.connection.url", url);
            props.setProperty("hibernate.connection.username", user);
            props.setProperty("hibernate.connection.password", password);
            props.setProperty("hibernate.dialect", dialect);
            props.setProperty("hibernate.connection.autocommit", "true");
            props.setProperty("hibernate.cache.provider_class", "org.hibernate.cache.NoCacheProvider");
            props.setProperty("hibernate.hbm2ddl.auto", "update");
            final AnnotationConfiguration configuration = new AnnotationConfiguration();
            configuration.addProperties(props);
            sessionFactory = configuration.addAnnotatedClass(MessageExchangeSnapshot.class)
                    .addAnnotatedClass(ComponentQueuesStatusSnapshot.class).addAnnotatedClass(
                            QueuesStatus.class).buildSessionFactory();
        } catch (final Throwable e) {
            throw new ExceptionInInitializerError(e);
        }
    }
}
