/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2008 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.tools.webadmin.datacollector.util;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.hibernate.HibernateException;
import org.hibernate.Session;
import org.ow2.petals.tools.webadmin.datacollector.bo.queues.ComponentQueuesStatusSnapshot;
import org.ow2.petals.tools.webadmin.datacollector.bo.queues.QueuesStatus;
import org.ow2.petals.tools.webadmin.datacollector.client.exception.DataCollectorClientException;
import org.ow2.petals.tools.webadmin.datacollector.client.service.TransportMonitoringService;
import org.ow2.petals.tools.webadmin.datacollector.dao.DAOException;
import org.ow2.petals.tools.webadmin.datacollector.dao.queues.ComponentQueuesStatusSnapshotDAO;

/**
 * @author mcarpentier
 * 
 *         Deals with all queues operations : retrieve, save,...
 * 
 */
public class Queuesmanager {

    public void collectQueues(String server, TransportMonitoringService tms)
            throws DataCollectorClientException, DAOException {
        Session session = null;
        try {
            session = HibernateHelper.getSession();
            final Map<String, ComponentQueuesStatusSnapshot> all = new HashMap<String, ComponentQueuesStatusSnapshot>();
            final List<String> transportersList = tms.getTransporters();
            final Date d = new Date();
            for (final String transporter : transportersList) {
                final Map<String, Integer> qSizes = tms.getQueueCurrentSizes(transporter);

                for (final String comp : qSizes.keySet()) {
                    ComponentQueuesStatusSnapshot c = null;
                    if (all.containsKey(comp)) {
                        c = all.get(comp);
                    } else {
                        c = new ComponentQueuesStatusSnapshot(d.getTime(), server, comp);
                        all.put(comp, c);
                    }
                    if (qSizes.get(comp) > 0) {
                        c.addQueueStatus(transporter, qSizes.get(comp));
                    }
                }
            }

            /** datas has been collected, persist queues status objects * */
            for (final String component : all.keySet()) {
                if ((all.get(component).getQueuesStatus() != null)
                        && (!all.get(component).getQueuesStatus().isEmpty())) {
                    ComponentQueuesStatusSnapshotDAO.getInstance().create(session,
                            all.get(component));
                }
            }
        } catch (HibernateException e) {
            if (session != null) {
                session.close();
            }
            throw new DataCollectorClientException("Can't connect to the database", e);
        } catch (DataCollectorConfigException e) {
            if (session != null) {
                session.close();
            }
            throw new DataCollectorClientException("Can't configure the database", e);
        } finally {
            session.close();
        }
    }

    public Map<String, Map<String, Long>> getAllCurrentComponentQueuesStatus(String server,
            TransportMonitoringService tms) throws DataCollectorClientException {

        final Map<String, Map<String, Long>> result = new HashMap<String, Map<String, Long>>();
        final List<String> transportersList = tms.getTransporters();
        final Map<String, ComponentQueuesStatusSnapshot> all = new HashMap<String, ComponentQueuesStatusSnapshot>();

        final Date d = new Date();
        for (final String transporter : transportersList) {
            final Map<String, Integer> qSizes = tms.getQueueCurrentSizes(transporter);

            for (final String comp : qSizes.keySet()) {
                ComponentQueuesStatusSnapshot c = null;
                if (all.containsKey(comp)) {
                    c = all.get(comp);
                } else {
                    c = new ComponentQueuesStatusSnapshot(d.getTime(), server, comp);
                    all.put(comp, c);
                }
                c.addQueueStatus(transporter, qSizes.get(comp));
            }
        }

        for (final String component : all.keySet()) {
            final Set<QueuesStatus> allQs = all.get(component).getQueuesStatus();
            final Map<String, Long> m = new HashMap<String, Long>();
            for (final QueuesStatus q : allQs) {
                m.put(q.getTransporter(), q.getValue());
            }
            result.put(component, m);
        }
        return result;
    }

    public Map<Long, Map<String, Long>> getComponentQueuesStatus(String server, String component)
            throws DAOException, HibernateException, DataCollectorConfigException {
        Session session = null;
        final Map<Long, Map<String, Long>> result = new HashMap<Long, Map<String, Long>>();
        try {
            session = HibernateHelper.getSession();
            final List<ComponentQueuesStatusSnapshot> list = ComponentQueuesStatusSnapshotDAO
                    .getInstance().retrieveByServerAndComponent(session, server, component);
            if (list != null) {
                for (final ComponentQueuesStatusSnapshot c : list) {
                    final Set<QueuesStatus> allQs = c.getQueuesStatus();
                    final Map<String, Long> m = new HashMap<String, Long>();
                    for (final QueuesStatus q : allQs) {
                        m.put(q.getTransporter(), q.getValue());
                    }
                    result.put(c.getTime(), m);
                }
            }
            return result;
        } catch (final DAOException e) {
            throw new DAOException("can't retrieve queues", e);
        } finally {
            if (session != null) {
                session.close();
            }
        }
    }
}
